<?php
/**
 * Demo file displaying a blog without using a template
 *
 * Note: This is a very advanced use of Quam Plures! Most of the time, calling
 * your blog through index.php (or a stub file) with a template will be enough.
 * You should try to customize a template before trying to use this file.
 *
 * Quam Plures - {@link http://quamplures.net/}
 * Released under GNU GPL License - {@link http://quamplures.net/license.html}
 * @copyright (c) 2009 - 2011 by the Quam Plures developers - {@link http://quamplures.net/}
 * @copyright (c)2003-2009 by Francois PLANQUE - {@link http://fplanque.net/}
 *
 * @todo EdB: figure out if this useful, shed if not else embrace and move to /qp_templates/
 *
 * @package templates
 */

// select which blog you want to display here
$blog = 1;
// force it to ignore the blog's designated template
$template = '';
// restrict posts to those published, thus hiding drafts.
$show_statuses = array();
// set a limit before which posts will be ignored
$timestamp_min = '';
// set a limit after which posts will be ignored
$timestamp_max = 'now';

# Tell Quam Plures not to redirect.
# This is necessary only if "301" for homepage is checked and the blog URL is
# set to something else than this page -- which is the case on demo installs.
# For production systems, properly set the blog URL then remove the line below:
$redir = 'no';

/**
 * Load Quam Plures configuration
 */
require_once dirname(__FILE__).'/qp_config/_config.php';

/**
 * Initialize Quam Plures
 *
 * Loads the blog identified by $blog with other params above
 */
require_once $inc_path.'_blog_main.inc.php';

// Make sure includes will check in the current folder!
$ads_current_template_path = dirname(__FILE__).'/';

// --------------------- PAGE LEVEL CACHING SUPPORT ---------------------
// This is totally optional. General caching must be enabled
// in Global settings, otherwise this will do nothing.
load_class( '_core/model/_pagecache.class.php' );
$PageCache = new PageCache( NULL );

// Check for cached content & Start caching if needed:
if( ! $PageCache->check() )
{
	template_init( $disp ); // do $disp-based initializations...
	add_js_for_toolbar(); // registers javascripts for the navbar
	header_content_type( 'text/html' ); // sets charset
	?>
	<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
	<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="<?php locale_lang(); ?>" lang="<?php locale_lang(); ?>">
	<head>
	<?php
	// spark the plugin manager
	$Plugins->trigger_event( 'TemplateBeginHtmlHead' );
	?>
	<title><?php // title for the current request
	request_title( array(
		'auto_pilot' => 'seo_title'
	) ); ?></title>
	<?php
	template_content_meta(); // Content-Type meta tag
	template_base_tag(); // for css and img includes
	template_description_tag(); // description meta tag
	template_keywords_tag(); // keywords meta tag
	robots_tag(); // robots meta tag
	meta_generator_tag(); // for QP's stats
	if( $Blog->get_setting( 'feed_content' ) != 'none' )
	{
		template_feed_tags();
	}
	?>
	<link rel="EditURI" type="application/rsd+xml" title="RSD" href="<?php echo $srvc_url; ?>rsd.php?blog=<?php echo $Blog->ID; ?>" />
	<meta name="viewport" content="width = 750" />
	<link rel="stylesheet" href="qp_rsc/css/standalone.css" type="text/css" />
	<?php
	include_headlines(); // output collected <head> lines
	?>
	</head>

	<body>
	<?php
	// spark the plugin manager
	$Plugins->trigger_event( 'TemplateBeginHtmlBody' );

	// --- --- --- --- --- --- --- --- --- --- ---
	// --- TOOLBAR INCLUDED HERE ---
	template_include( '_toolbar.inc.php' );

	if( is_logged_in() )
	{
		echo "\n".'<div id="template_wrapper" class="template_wrapper_loggedin">'."\n";
	}
	else
	{
		echo "\n".'<div id="template_wrapper" class="template_wrapper_anonymous">'."\n";
	}
	?>

	<div class="pageHeader">
	<?php
	// display the Public Blog List widget
	template_widget( array(
		'widget' => 'bloglist',
		'block_start' => '<div class="bloglist">',
		'block_end' => '</div>',
		'block_display_title' => false,
		'list_start' => '',
		'list_end' => '',
		'item_start' => '',
		'item_end' => '',
		'item_selected_start' => '',
		'item_selected_end' => '',
		'link_selected_class' => 'buttons',
		'link_default_class' => 'buttons',
	) );
	?>

	<div class="pageTitle">
	<h1 id="pageTitle"><?php $Blog->disp( 'name', 'htmlbody' ) ?></h1>
	</div>

	</div>

	<div class="pageSubTitle"><?php $Blog->disp( 'tagline', 'htmlbody' ) ?></div>

	<div class="main">

	<div class="bPosts">
	<?php
	// messages generated from actions
	messages( array() );

	// prev/next links in single post mode
	item_prevnext_links( array(
		'block_start' => '<table class="prevnext_post"><tr>',
		'prev_start' => '<td>',
		'prev_end' => '</td>',
		'next_start' => '<td class="right">',
		'next_end' => '</td>',
		'block_end' => '</tr></table>',
	) );

	// title for the current request
	request_title( array(
		'title_before' => '<h2>',
		'title_after' => '</h2>',
	) );

	// prev/next links on multi-post pages
	mainlist_page_links( array(
		'block_start' => '<p class="center"><strong>',
		'block_end' => '</strong></p>',
	) );

	// display a message if list is empty
	display_if_empty( array() );

	// loop through as long as we have an Item...
	while( $Item = & mainlist_get_item() )
	{
		// display the date if it changed
		$MainList->date_if_changed( array() );


		// --- --- --- --- --- --- --- --- --- --- ---
		// --- POST CONTENT INCLUDED HERE ---
		template_include( '_item_block.inc.php', array() );

	}

	// prev/next links on multi-post pages
	mainlist_page_links( array(
		'block_start' => '<p class="center"><strong>',
		'block_end' => '</strong></p>',
	) );

	// --- --- --- --- --- --- --- --- --- --- ---
	// --- MAIN CONTENT INCLUDED HERE ---
	template_include( '$disp$', array(
		// these cases already handled by this file
		'disp_posts' => '',
		'disp_single' => '',
		'disp_page' => '',
	) );
	?>
	</div>

	<div class="bSideBar">

	<div class="bSideItem">
	<h3><?php $Blog->disp( 'name', 'htmlbody' ); ?></h3>
	<p>
	<?php
	// BLOG LONG DESCRIPTION:
	$Blog->disp( 'longdesc', 'htmlbody' );
	?>
	</p>
	<?php
	// prev/next links on multi-post pages
	mainlist_page_links( array(
		'block_start' => '<p class="center"><strong>',
		'block_end' => '</strong></p>',
		'links_format' => '$prev$ :: $next$',
		'prev_text' => '&lt;&lt; '.T_('Previous'),
		'next_text' => T_('Next').' &gt;&gt;',
	) );

	// display the Common Links widget
	template_widget( array(
		'widget' => 'common_links',
		'show_recently' => true,
		'show_archives' => true,
		'show_categories' => false,
		'show_latestcomments' => false,
	) );

	// display the Calendar plugin (if installed)
	$Plugins->call_by_code( 'calendar', array(
		'block_start' => '',
		'block_end' => '',
		'displaycaption' => true,
		'linktomontharchive' => false,
	) );
	?>
	</div>

	<?php
	// display the Search Form widget
	template_widget( array(
		'widget' => 'search_form',
		'block_start' => '<div class="bSideItem">',
		'block_end' => '</div>',
		'block_title_start' => '<h3 class="sideItemTitle">',
		'block_title_end' => '</h3>',
	) );

	// display the Category List widget
	template_widget( array(
		'widget' => 'category_list',
		'block_start' => '<div class="bSideItem">',
		'block_end' => '</div>',
		'block_title_start' => '<h3 class="sideItemTitle">',
		'block_title_end' => '</h3>',
	) );

	// display the Archives plugin (if installed)
	$Plugins->call_by_code( 'archives', array(
		'block_start' => '<div class="bSideItem">',
		'block_end' => '</div>',
		'block_title_start' => '<h3>',
		'block_title_end' => '</h3>',
	) );

	// display the Linkblog widget
	template_widget( array(
		'widget' => 'linkblog',
		'block_start' => '<div class="bSideItem">',
		'block_end' => '</div>',
		'block_title_start' => '<h3 class="sideItemTitle">',
		'block_title_end' => '</h3>',
	) );

	// display the User Tools widget
	template_widget( array(
		'widget' => 'user_tools',
		'block_start' => '<div class="bSideItem">',
		'block_end' => '</div>',
		'block_title_start' => '<h3 class="sideItemTitle">',
		'block_title_end' => '</h3>',
	) );

	// display the XML Feeds widget
	template_widget( array(
		'widget' => 'xml_feeds',
		'block_start' => '<div class="bSideItem">',
		'block_end' => '</div>',
		'block_title_start' => '<h3 class="sideItemTitle">',
		'block_title_end' => '</h3>',
	) );
	?>
	</div>

	</div>

	<div class="footer">
	<?php echo sprintf( T_('This is a demo page for <a href="%s">%s</a>.'), $app_homepage, $app_name ); ?>
	</div>

	</div>
	</body>
	</html>
	<?php
	// save collected cached data (if needed)
	$PageCache->end_collect();
}
$Hit->log(); // log the hit on this page ?>
