<?php
/**
 * This file implements the UI for file browsing.
 *
 * This file is part of Quam Plures - {@link http://quamplures.net/}
 * See also {@link https://launchpad.net/quam-plures}.
 *
 * @copyright (c) 2009 - 2011 by the Quam Plures developers - {@link http://quamplures.net/}
 * @copyright (c)2003-2009 by Francois PLANQUE - {@link http://fplanque.net/}
 * Parts of this file are copyright (c)2004-2006 by Daniel HAHLER - {@link http://thequod.de/contact}.
 *
 * {@internal License choice
 * - If you have received this file as part of a package, please find the license.txt file in
 *   the same folder or the closest folder above for complete license terms.
 * - If you have received this file individually (e-g: from http://evocms.cvs.sourceforge.net/)
 *   then you must choose one of the following licenses before using the file:
 *   - GNU General Public License 2 (GPL) - http://www.opensource.org/licenses/gpl-license.php
 *   - Mozilla Public License 1.1 (MPL) - http://www.opensource.org/licenses/mozilla1.1.php
 * }}
 *
 * {@internal Open Source relicensing agreement:
 * Daniel HAHLER grants Francois PLANQUE the right to license
 * Daniel HAHLER's contributions to this file and the b2evolution project
 * under any OSI approved OSS license (http://www.opensource.org/licenses/).
 * }}
 *
 * {@internal Below is a list of authors who have contributed to design/coding of this file: }}
 * @author blueyed: Daniel HAHLER.
 * @author fplanque: Francois PLANQUE.
 *
 * @package files
 */
if( !defined('QP_MAIN_INIT') ) die( 'Please, do not access this page directly.' );

/**
 * @var Filelist
 */
global $fm_Filelist;
/**
 * fp> Temporary. I need this for NuSphere debugging.
 * @var File
 */
global $lFile;
/**
 * @var string
 */
global $fm_flatmode;
/**
 * @var User
 */
global $current_User;
/**
 * @var UserSettings
 */
global $UserSettings;
/**
 * @var Log
 */
global $Messages;
/**
 * @var Filelist
 */
global $selected_Filelist;
/**
 * @var Item
 */
global $edited_Item;

global $edited_User;

global $Blog;

global $fm_hide_dirtree, $create_name, $ads_list_path;

// Name of the iframe we want some atiosn to come back to:
global $iframe_name;
?>

<!-- FILE LIST -->

<?php
$Form = new Form( NULL, 'FilesForm', 'post', 'none' );
$Form->begin_form();
$Form->hidden_ctrl();
$Form->hidden( 'confirmed', '0' );
$Form->hidden( 'md5_filelist', $fm_Filelist->md5_checksum() );
$Form->hidden( 'md5_cwd', md5($fm_Filelist->get_ads_list_path()) );
$Form->hiddens_by_key( get_memorized('fm_selected') ); // 'fm_selected' gets provided by the form itself
?>
<table class="filelist">
<thead>
<?php
/*****************  Col headers  ****************/
echo '<tr>';

// "Go to parent" icon
echo '<th class="firstcol">';
if( empty($fm_Filelist->_rds_list_path) )
{ // cannot go higher
		echo '&nbsp;';	// for IE
}
else
{
	echo action_icon( T_('Go to parent folder'), 'folder_parent', regenerate_url( 'path', 'path='.$fm_Filelist->_rds_list_path.'..' ) );
}
echo '</th>';

echo '<th class="nowrap">';
if( $UserSettings->get( 'fm_imglistpreview' ) )
{ // Image file preview:
	$col_title = T_('Icon/Type');
}
else
{
	$col_title = /* TRANS: short for (file)Type */ T_(' T ');
}
echo $fm_Filelist->get_sort_link( 'type', $col_title );
echo '</th>';

if( $fm_flatmode )
{
	echo '<th>'.$fm_Filelist->get_sort_link( 'path', /* TRANS: file/directory path */ T_('Path') ).'</th>';
}

echo '<th class="nowrap">'.$fm_Filelist->get_sort_link( 'name', /* TRANS: file name */ T_('Name') ).'</th>';

if( $UserSettings->get('fm_showtypes') )
{ // Show file types column
	echo '<th class="nowrap">'.$fm_Filelist->get_sort_link( 'type', /* TRANS: file type */ T_('Type') ).'</th>';
}

echo '<th class="nowrap">'.$fm_Filelist->get_sort_link( 'size', /* TRANS: file size */ T_('Size') ).'</th>';

if( $UserSettings->get('fm_showdate') != 'no' )
{ // Show last mod column
	echo '<th class="nowrap">'.$fm_Filelist->get_sort_link( 'lastmod', /* TRANS: file's last change / timestamp */ T_('Last change') ).'</th>';
}

$popfor = param( 'popfor', 'string', '', true );

if( $popfor == '' )
{
	if( $UserSettings->get('fm_showfsperms') )
	{ // Show file perms column
		echo '<th class="nowrap">'.$fm_Filelist->get_sort_link( 'perms', /* TRANS: file's permissions (short) */ T_('Perms') ).'</th>';
	}

	if( $UserSettings->get('fm_showfsowner') )
	{ // Show file owner column
		echo '<th class="nowrap">'.$fm_Filelist->get_sort_link( 'fsowner', /* TRANS: file owner */ T_('Owner') ).'</th>';
	}

	if( $UserSettings->get('fm_showfsgroup') )
	{ // Show file group column
		echo '<th class="nowrap">'.$fm_Filelist->get_sort_link( 'fsgroup', /* TRANS: file group */ T_('Group') ).'</th>';
	}

	echo '<th class="lastcol nowrap">'. /* TRANS: file actions; edit, rename, copy, .. */ T_('Actions').'</th>';
}
echo '</tr>';
?>
</thead>
<tbody>

<?php
param( 'checkall', 'integer', 0 );  // Non-Javascript-CheckAll

/***********************************************************/
/*                    MAIN FILE LIST:                      */
/***********************************************************/
$countFiles = 0;
while( $lFile = & $fm_Filelist->get_next() )
{ // Loop through all Files:
	echo '<tr class="'.( $countFiles%2 ? 'odd' : 'even' ).'">';

	/********************    Checkbox:    *******************/
	echo '<td class="checkbox firstcol">';
	echo '<span name="surround_check" class="checkbox_surround_init">';
	echo '<input title="'.T_('Select this file').'" type="checkbox" class="checkbox"
		name="fm_selected[]" value="'.rawurlencode($lFile->get_rdfp_rel_path()).'" id="cb_filename_'.$countFiles.'"';
	global $checkall;
	if( $checkall || $selected_Filelist->contains( $lFile ) )
	{
		echo ' checked="checked"';
	}
	echo ' />';
	echo '</span>';

	/***********  Hidden info used by Javascript:  ***********/

	global $mode;
	if( $mode == 'upload' )
	{	// This mode allows inserting images or links to files in items...
		$lFile->load_meta( true );
		echo '<input type="hidden" name="link_'.$countFiles.'" id="link_'.$countFiles.'" value="'
			.$lFile->get_url().'" />'."\n";
		// this is the title
		echo '<input type="hidden" name="title_'.$countFiles.'" id="title_'.$countFiles.'" value="'
			.$lFile->dget('title', 'htmlattr').'" />'."\n";
		// this is the alttxt
		echo '<input type="hidden" name="alttxt_'.$countFiles.'" id="alttxt_'.$countFiles.'" value="'
			.$lFile->dget('alt', 'htmlattr').'" />'."\n";
		// this is the caption
		echo '<input type="hidden" name="caption_'.$countFiles.'" id="caption_'.$countFiles.'" value="'
			.$lFile->dget('desc', 'htmlattr').'" />'."\n";
		// this is width and height OR false for non-image files
		echo '<input type="hidden" name="dimens_'.$countFiles.'" id="dimens_'.$countFiles.'" value="'
			.$lFile->get_image_size().'" />'."\n";
	}

	echo '</td>';

	/********************  Icon / File type:  *******************/
	echo '<td class="icon_type">';
	if( $UserSettings->get( 'fm_imglistpreview' ) )
	{	// Image preview OR full type:
		if( $lFile->is_dir() )
		{ // Navigate into Directory
			echo '<a href="'.$lFile->get_view_url().'" title="'.T_('Change into this directory').'">'.$lFile->get_icon().' '.T_('Directory').'</a>';
		}
		else
		{
			echo $lFile->get_preview_thumb( 'fulltype' );
		}
	}
	else
	{	// No image preview, small type:
 		if( $lFile->is_dir() )
		{ // Navigate into Directory
			echo '<a href="'.$lFile->get_view_url().'" title="'.T_('Change into this directory').'">'.$lFile->get_icon().'</a>';
		}
		else
		{ // File
			echo $lFile->get_view_link( $lFile->get_icon(), NULL, $lFile->get_icon() );
		}
	}
	echo '</td>';

	/*******************  Path (flatmode): ******************/
	if( $fm_flatmode )
	{
		echo '<td class="filepath">';
		echo dirname($lFile->get_rdfs_rel_path()).'/';
		echo '</td>';
	}

	echo '<td class="fm_filename">';

	/*************  Invalid filename warning:  *************/
	if( !$lFile->is_dir() )
	{
		if( $error_filename = validate_filename( $lFile->get_name() ) )
		{ // TODO: Warning icon with hint
			echo get_icon( 'warning', 'imgtag', array( 'class' => 'filenameIcon', 'title' => $error_filename ) );
		}
	}
	elseif( $error_dirname = validate_dirname( $lFile->get_name() ) )
	{ // TODO: Warning icon with hint
		echo get_icon( 'warning', 'imgtag', array( 'class' => 'filenameIcon', 'title' => $error_dirname ) );
	}

	/****  Open in a new window  (only directories)  ****/
	if( $lFile->is_dir() )
	{ // Directory
		$browse_dir_url = $lFile->get_view_url();
		$popup_url = url_add_param( $browse_dir_url, 'mode=popup' );
		$target = 'evo_fm_'.$lFile->get_md5_ID();

		echo '<a href="'.$browse_dir_url.'" target="'.$target.' " class="filenameIcon"
			title="'.T_('Open in a new window').'" onclick="'
			."return pop_up_window( '$popup_url', '$target' )"
			.'">'.get_icon( 'window_new' ).'</a>';
	}

	/***************  Link ("chain") icon:  **************/
	// if( ! $lFile->is_dir() )	// fp> OK but you need to include an else:placeholder, otherwise the display is ugly
	{	// Only provide link/"chain" icons for files.
		// TODO: dh> provide support for direcories (display included files).

		// fp> here might not be the best place to put the perm check
		if( isset($edited_Item) && $current_User->check_perm( 'item_post!CURSTATUS', 'edit', false, $edited_Item ) )
		{	// Offer option to link the file to an Item (or anything else):
			$link_attribs = array();
			$link_action = 'link';
			if( $mode == 'upload' )
			{	// We want the action to happen in the post attachments iframe:
				$link_attribs['target'] = $iframe_name;
				$link_action = 'link_inpost';
			}
			echo action_icon( T_('Link this file!'), 'link',
				regenerate_url( 'fm_selected', 'action='.$link_action.'&amp;fm_selected[]='.rawurlencode($lFile->get_rdfp_rel_path()) ),
				NULL, NULL, NULL, $link_attribs );
				echo ' ';
		}

		if( isset($edited_User) ) // fp> Perm already checked in controller
		{	// Offer option to link the file to an Item (or anything else):
			echo action_icon( T_('Link this file!'), 'link',
				regenerate_url( 'fm_selected', 'action=link_user&amp;fm_selected[]='.rawurlencode($lFile->get_rdfp_rel_path()) ),
				NULL, NULL, NULL, array() );
				echo ' ';
		}

	}

	/********************  Filename  ********************/
	if( $lFile->is_dir() )
	{ // Directory
		// Link to open the directory in the curent window
		echo '<a href="'.$browse_dir_url.'">'.$lFile->dget('name').'</a>';
	}
	else
	{ // File
		if( $view_link = $lFile->get_view_link( $lFile->get_name(), NULL, NULL ) )
		{
			echo $view_link;
		}
		else
		{ // File extension unrecognized
			echo $lFile->dget('name');
		}
	}

	/***************  File meta data:  **************/
	echo '<span class="filemeta">';

	// does the user want image pixel size info?
	if( $UserSettings->get( 'fm_getimagesizes' ) )
	{
		echo ' ('.$lFile->get_image_size( 'widthxheight' ).')';
	}

	// if we have loaded metadata use the caption as a title
	if( $lFile->meta == 'loaded' )
	{
		// display file title
		if( $lFile->title != '' )
		{
			echo ' <span class="tooltitle" title="'.format_to_output( $lFile->title, 'htmlattr' ).'">'.T_('(TITLE)').'</span>';
		}
		else
		{
			echo ' '.T_('(no title)');
		}
		// display file alt text
		if( $lFile->alt != '' )
		{
			echo ' <span class="tooltitle" title="'.format_to_output( $lFile->alt, 'htmlattr' ).'">'.T_('(ALT TEXT)').'</span>';
		}
		else
		{
			echo ' '.T_('(no alt text)');

		}
		// begin a tag for the file caption
		if( $lFile->desc != '' )
		{
			echo ' <span class="tooltitle" title="'.format_to_output( $lFile->desc, 'htmlattr' ).'">'.T_('(DESCRIPTION)').'</span>';
		}
		else
		{
			echo ' '.T_('(no description)');
		}
	}

	echo '</span>';
	echo '</td>';

	/*******************  File type  ******************/
	if( $UserSettings->get('fm_showtypes') )
	{ // Show file types
		echo '<td class="type">'.$lFile->get_type().'</td>';
	}

	/*******************  File size  ******************/
	echo '<td class="size">'.$fm_Filelist->get_File_size_formatted($lFile).'</td>';

	/****************  File time stamp  ***************/
	if( $UserSettings->get('fm_showdate') != 'no' )
	{ // Show last modified datetime (always full in title attribute)
		$lastmod_date = $lFile->get_lastmod_formatted( 'date' );
		$lastmod_time = $lFile->get_lastmod_formatted( 'time' );
		echo '<td class="timestamp" title="'.$lastmod_date.' '.$lastmod_time.'">';
		if( $UserSettings->get('fm_showdate') == 'long' )
		{
			echo '<span class="date">'.$lastmod_date.'</span> ';
			echo '<span class="time">'.$lastmod_time.'</span>';
		}
		else
		{	// Compact format
			echo $lFile->get_lastmod_formatted( 'compact' );
		}
		echo '</td>';
	}

	/****************  File pemissions  ***************/
	if( $popfor == '' )
	{
		if( $UserSettings->get('fm_showfsperms') )
		{ // Show file perms
			echo '<td class="perms">';
			$fm_permlikelsl = $UserSettings->param_Request( 'fm_permlikelsl', 'fm_permlikelsl', 'integer', 0 );
			if( $current_User->check_perm( 'files', 'edit' ) )
			{ // User can edit:
				echo '<a title="'.T_('Edit permissions').'" href="'.regenerate_url( 'fm_selected,action', 'action=edit_perms&amp;fm_selected[]='.rawurlencode($lFile->get_rdfp_rel_path()) ).'">'
					.$lFile->get_perms( $fm_permlikelsl ? 'lsl' : '' ).'</a>';
			}
			else
			{
				echo $lFile->get_perms( $fm_permlikelsl ? 'lsl' : '' );
			}
			echo '</td>';
		}

		/****************  File owner  ********************/
		if( $UserSettings->get('fm_showfsowner') )
		{ // Show file owner
			echo '<td class="fsowner">';
			echo $lFile->get_fsowner_name();
			echo '</td>';
		}

		/****************  File group *********************/
		if( $UserSettings->get('fm_showfsgroup') )
		{ // Show file owner
			echo '<td class="fsgroup">';
			echo $lFile->get_fsgroup_name();
			echo '</td>';
		}

		/*****************  Action icons  ****************/
		echo '<td class="actions lastcol">';
		if( $current_User->check_perm( 'files', 'edit' ) )
		{ // User can edit:
			if( $lFile->is_editable( $current_User->check_perm( 'files', 'all' ) ) )
			{
				echo action_icon( T_('Edit file...'), 'edit', regenerate_url( 'fm_selected', 'action=edit_file&amp;fm_selected[]='.rawurlencode($lFile->get_rdfp_rel_path()) ) );
			}
			else
			{
				echo get_icon( 'edit', 'noimg' );
			}
		}
		echo action_icon( T_('Edit properties...'), 'properties', regenerate_url( 'fm_selected', 'action=edit_properties&amp;fm_selected[]='.rawurlencode($lFile->get_rdfp_rel_path()) ) );

		if( $current_User->check_perm( 'files', 'edit' ) )
		{ // User can edit:
			echo action_icon( T_('Rename'), 'rename', regenerate_url( 'fm_selected', 'action=rename&amp;fm_selected[]='.rawurlencode($lFile->get_rdfp_rel_path()) ) );
			echo action_icon( T_('Move'), 'move', regenerate_url( 'fm_mode,fm_sources,fm_sources_root', 'fm_mode=file_move&amp;fm_sources[]='.rawurlencode( $lFile->get_rdfp_rel_path() ).'&amp;fm_sources_root='.$fm_Filelist->_FileRoot->ID ) );
			echo action_icon( T_('Copy'), 'copy', regenerate_url( 'fm_mode,fm_sources,fm_sources_root', 'fm_mode=file_copy&amp;fm_sources[]='.rawurlencode( $lFile->get_rdfp_rel_path() ).'&amp;fm_sources_root='.$fm_Filelist->_FileRoot->ID ) );
			echo action_icon( T_('Delete'), 'delete', regenerate_url( 'fm_selected', 'action=delete&amp;fm_selected[]='.rawurlencode( $lFile->get_rdfp_rel_path() ) ) );
		}
		echo '</td>';
	}

	echo '</tr>';

	$countFiles++;
}
// End of file list..

/**
 * @global integer Number of cols for the files table, 6 is minimum.
 */
$filetable_cols = 5
	+ (int)$fm_flatmode
	+ (int)$UserSettings->get('fm_showtypes')
	+ (int)($UserSettings->get('fm_showdate') != 'no')
	+ (int)$UserSettings->get('fm_showfsperms')
	+ (int)$UserSettings->get('fm_showfsowner')
	+ (int)$UserSettings->get('fm_showfsgroup')
	+ (int)$UserSettings->get('fm_imglistpreview');

if( $countFiles == 0 )
{ // Filelist errors or "directory is empty"
	?>
	<tr>
	<td class="firstcol">&nbsp;</td> <?php /* blueyed> This empty column is needed so that the defaut width:100% style of the main column below makes the column go over the whole screen */ ?>
	<td class="lastcol" colspan="<?php echo $filetable_cols - 1 ?>" id="fileman_error">
	<?php
	if( ! $Messages->count('fl_error') )
	{ // no Filelist errors, the directory must be empty
		$Messages->add( T_('No files found.')
			.( $fm_Filelist->is_filtering() ? '<br />'.T_('Filter').': &laquo;'.$fm_Filelist->get_filter().'&raquo;' : '' ), 'fl_error' );
	}
	$Messages->display( '', '', true, 'fl_error', 'log_error' );
	?>
	</td>
	</tr>

	<?php
}
else
{
	// -------------
	// Footer with "check all", "with selected: ..":
	// --------------
	if( $popfor != 'attach' )
	{
		?>
		<tr class="listfooter firstcol lastcol">
		<td colspan="<?php echo $filetable_cols ?>">
		<?php
		echo $Form->check_all();

		$field_options = array();
		$title_options = array();
		$caption_options = array();

		if( $mode != 'upload' && ($fm_Filelist->get_root_type() == 'collection' || !empty($Blog)) )
		{	// We are browsing files for a collection:
			// fp> TODO: use current as default but let user choose into which blog he wants to post
			$field_options['make_post'] = T_('Make one post (including all images)');
			$field_options['make_posts'] = T_('Make multiple posts (1 per image)');
			$title_options['default'] = T_('include if provided');
			$caption_options['default'] = T_('include if provided');
		}

		if( $mode == 'upload' )
		{ // we are in a popup opened by the "Files" button
			$field_options['float_left'] = T_('Float left');
			$field_options['float_right'] = T_('Float right');
			$field_options['centered'] = T_('Centered');
			$field_options['inline'] = T_('In-line');
			$title_options['default'] = T_('default');
			$title_options['placeholder'] = T_('placeholder');
			$title_options['none'] = T_('no title');
			$caption_options['default'] = T_('default');
			$caption_options['placeholder'] = T_('placeholder');
			$caption_options['none'] = T_('no caption');
		}

		if( $popfor == '' )
		{
			if( $current_User->check_perm( 'files', 'edit' ) )
			{ // current user can edit files
				$field_options['rename'] = T_('Rename files...');
				$field_options['delete'] = T_('Delete files...');
				// NOTE: No delete confirmation by javascript, we need to check DB integrity!
			}
			// BROKEN ?
			$field_options['download'] = T_('Download files as ZIP archive...');

			/* Not fully functional:
			$field_options['copy'] = T_('Copy the selected files...');
			$field_options['move'] = T_('Move the selected files...');
			*/
		}

		$Form->switch_layout( 'none' );
		$Form->select_input_array( 'group_action', $action, $field_options, ' || <strong>'.T_('With selected files').'</strong>' );

		if( $mode == 'upload' )
		{ // we are in a popup opened by the "Files" button
			$Form->select_input_array( 'title_action', 'default', $title_options, ' <strong>'.T_('title').'</strong>' );
			$Form->select_input_array( 'caption_action', 'default', $caption_options, ' <strong>'.T_('caption').'</strong>' );
		}
		// The "go" button gives us what to do with the file, title, and caption for all files
		$Form->submit_input( array( 'name'=>'actionArray[group_action][title_action][caption_action]', 'value'=>T_('Go!'), 'onclick'=>'return js_act_on_selected();' ) );
		$Form->switch_layout( NULL );
		?>
		</td>
		</tr>
		<?php
	}
}
?>
</tbody>
</table>
<?php
$Form->end_form();

// allow a plugin to provide it's own $file_tags array
global $Plugins;
if( $plugin_return = $Plugins->trigger_event_first_return( 'ItemInsertFileJavascript' ) )
{
	$editor_Plugin = & $Plugins->get_by_ID( $plugin_return['plugin_ID'] );
	$file_tags = $plugin_return['plugin_return'];
}
// if we don't have one, we need one
if( ! isset( $file_tags ) )
{
	$file_tags = array(
		'insert_tag' => 'span',
		'insert_class_floats' => 'FLOATDIRECTION center',
		'insert_class_centered' => 'image_block',
		'insert_style_floats' => 'STYLEWIDTH',
		'insert_style_centered' => '',
		'title_tag' => 'span',
		'title_class' => 'image_title',
		'title_style' => '',
		'title_post' => '<br />',
		'image_pre' => '',
		'image_class' => 'center',
		'image_style' => '',
		'image_post' => '',
		'caption_pre' => '<br />',
		'caption_tag' => 'span',
		'caption_class' => 'image_legend',
		'caption_style' => '',
		'caption_post' => '',
		'caption_none' => '',
		'caption_separator' => '',
	);
}

/**
 * @todo (legacy): remove these javascript functions to an external .js file and include them through add_headline()
 */
if( $countFiles )
{{{ // include JS
	?>
	<script type="text/javascript">
	<!--
	/**
	 * Prepare for inserting images or file links into the item/post
	 */
	function js_act_on_selected()
	{
		// There may be an easier selector than below but couldn't make sense of it :(
		disp_value = jQuery('#group_action option:selected').attr('value');
		title_action = jQuery('#title_action option:selected').attr('value');
		caption_action = jQuery('#caption_action option:selected').attr('value');

		if( disp_value == 'float_left' )
		{
			insert_tag_for_selected_files( 'float', 'left', title_action, caption_action );
			return false;
		}
		if( disp_value == 'float_right' )
		{
			insert_tag_for_selected_files( 'float', 'right', title_action, caption_action );
			return false;
		}
		if( disp_value == 'centered' )
		{
			insert_tag_for_selected_files( 'center', 'center', title_action, caption_action );
			return false;
		}
		if( disp_value == 'inline' )
		{
			insert_tag_for_selected_files( 'inline', 'inline', title_action, caption_action );
			return false;
		}

		// other actions:
		return true;
	}

	/**
	 * Check if files are selected. This should be used as "onclick" handler for
	 * "With selected" actions (onclick="return check_if_selected_files();").
	 * @return boolean true, if something is selected, false if not.
	 */
	function check_if_selected_files()
	{
		elems = document.getElementsByName( 'fm_selected[]' );
		var checked = 0;
		for( i = 0; i < elems.length; i++ )
		{
			if( elems[i].checked )
			{
				checked++;
			}
		}
		if( !checked )
		{
			alert( '<?php echo TS_('Nothing selected.') ?>' );
			return false;
		}
		else
		{
			return true;
		}
	}

	/**
	 * Insert IMG tags into parent window for selected files
	 */
	function insert_tag_for_selected_files( what, where, title_action, caption_action )
	{
		var elems = document.getElementsByName( 'fm_selected[]' );
		var snippet = '';

		// these values might come from an editor plugin :)
		var insert_tag = '<?php echo $file_tags['insert_tag']; ?>';
		if( what == 'float' )
		{
			var insert_class = '<?php echo $file_tags['insert_class_floats']; ?>';
			var insert_style = '<?php echo $file_tags['insert_style_floats']; ?>';
		}
		else if( what == 'center' )
		{
			var insert_class = '<?php echo $file_tags['insert_class_centered']; ?>';
			var insert_style = '<?php echo $file_tags['insert_style_centered']; ?>';
		}
		var title_open = '<?php echo $file_tags['title_tag']; ?>';
		var title_class = '<?php echo $file_tags['title_class']; ?>';
		var title_style = '<?php echo $file_tags['title_style']; ?>';
		var title_post = '<?php echo $file_tags['title_post']; ?>';
		var image_pre = '<?php echo $file_tags['image_pre']; ?>';
		var image_class = '<?php echo $file_tags['image_class']; ?>';
		var image_style = '<?php echo $file_tags['image_style']; ?>';
		var image_post = '<?php echo $file_tags['image_post']; ?>';
		var caption_pre = '<?php echo $file_tags['caption_pre']; ?>';
		var caption_tag = '<?php echo $file_tags['caption_tag']; ?>';
		var caption_class = '<?php echo $file_tags['caption_class']; ?>';
		var caption_style = '<?php echo $file_tags['caption_style']; ?>';
		var caption_post = '<?php echo $file_tags['caption_post']; ?>';
		var caption_none = '<?php echo $file_tags['caption_none']; ?>';
		var caption_separator = '<?php echo $file_tags['caption_separator']; ?>';

		var snippet = ''; // this is what we will eventually insert into the item
		for( i = 0; i < elems.length; i++ )
		{
			if( elems[i].checked )
			{
				id = elems[i].id.substring( elems[i].id.lastIndexOf('_')+1, elems[i].id.length );
				link_value = document.getElementById( 'link_'+id ).value;
				title_value = document.getElementById( 'title_'+id ).value;
				alttxt_value = document.getElementById( 'alttxt_'+id ).value;
				caption_value = document.getElementById( 'caption_'+id ).value;
				dimens_value = document.getElementById( 'dimens_'+id ).value;

				// ensure we have both a title_value and an alttxt_value
				var link_array = link_value.split( "/" );
				var filename = link_array.pop();

				// dimens_value effectively tells us if we have an image or not
				if( dimens_value == '' )
				{ // this is a non-image ... and it needs a lot of help but not now. images first...
					
					if( title_value == '' ) { title_value = filename; }
					if( alttxt_value == '' ) { alttxt_value = '<?php echo TS_('Link to ') ?>' + filename; }
					snippet += '<a href="' + link_value + '" title="' + alttxt_value + '">' + title_value + '</a> ';
				}
				else
				{ // THIS IS AN IMAGE FILE - WOOHOO!
					// prep work - set some variables we'll need along the way
					var dims_arr = dimens_value.split( "x" ); // this is the image width
					var calculated_width = Math.round( parseFloat( dims_arr[0] ) + 12 );
					var image_tag = ''; // for first build
					var begin_block = ''; // for second build
					var end_block = ''; // for second build
					var title_html = ''; // for third build
					var caption_html = ''; // for fourth build

					// FIRST BUILD: the image tag. inserts, floats, and centered will all use the image tag
					image_tag = '<img src="' + link_value + '"';
					if( alttxt_value == '' ) { alttxt_value = filename; }
					image_tag += ' alt="' + alttxt_value + '"';
					image_tag += ' width="' + dims_arr[0] + '" height="' + dims_arr[1] + '"';
					if( image_class != '' ) { image_tag += ' class="' + image_class + '"'; }
					if( what != 'inline' && image_style != '' ) { image_tag += ' style="' + image_style + '"'; }
					image_tag += ' />';
					// for floats and centered we might have pre- and post- stuff to add
					if( what == 'float' || what == 'center' )
					{
						image_tag = image_pre + image_tag + image_post;
					}

					// SECOND BUILD: the overall insertion block (only floats and centered)
					if( what == 'float' || what == 'center' )
					{
						begin_block = '<' + insert_tag;
						if( insert_class != '' ) { begin_block += ' class="' + insert_class + '"'; }
						if( insert_style != '' ) { begin_block += ' style="' + insert_style + '"'; }
						begin_block += '>';
						var splitblocktag = insert_tag.split(" ");
						end_block = '</' + splitblocktag[0] + '>';

						// THIRD BUILD: a title block if desired (only floats and centered)
						if( ( title_action == 'default' && title_value != '' ) || title_action == 'placeholder' )
						{ // we want the title and have one, or we want a placeholder
							title_html = '<' + title_open;
							if( title_class != '' ) { title_html += ' class="' + title_class + '"'; }
							if( title_style != '' ) { title_html += ' style="' + title_style + '"'; }
							title_html += '>';
							if( title_action == 'placeholder' ) { title_value = '<?php echo TS_('TITLE PLACEHOLDER') ?>'; }
							var splittitletag = title_open.split(" ");
							title_html += title_value + '</' + splittitletag[0] + '>';
							title_html += title_post;
						}

						// FOURTH BUILD: a caption block if desired
						if( ( caption_action == 'default' && caption_value != '' ) || caption_action == 'placeholder' )
						{ // we want the caption and have one, or we want a placeholder
							caption_html = caption_pre + '<' + caption_tag;
							if( caption_class != '' ) { caption_html += ' class="' + caption_class + '"'; }
							if( caption_style != '' ) { caption_html += ' style="' + caption_style + '"'; }
							caption_html += '>';
							if( caption_action == 'placeholder' ) { caption_value = '<?php echo TS_('CAPTION PLACEHOLDER') ?>'; }
							var splitcaptiontag = caption_tag.split(" ");
							caption_html += caption_value + '</' + splitcaptiontag[0] + '>';
							caption_html += caption_post + caption_separator;
						}
						else
						{ // we need the "splitter" for TinyMCE and maybe others one day
							caption_html = caption_none + caption_separator;
						}

					} // end floats and centered

					// build the snippet and replace some things...
					snippet += begin_block + title_html + image_tag + caption_html + end_block + ' ';
					snippet = snippet.replace( /FLOATDIRECTION/g, 'float' + where );
					snippet = snippet.replace( /STYLEWIDTH/g, 'width:' + calculated_width + 'px;' );
					snippet = snippet.replace( /WIDTHONLY/g, calculated_width );

				} // end 'this is an image'

			} // end elems.checked

		} // end for each elem

		// check it...
		if( ! snippet.length )
		{ // eh?
			alert( '<?php echo TS_('You must select at least one file!') ?>' );
			return false;
		}
		else
		{ // ship it!
			if (! (window.focus && window.opener))
			{
				self.close();
				return true;
			}
			window.opener.focus();
			textarea_wrap_selection( window.opener.document.getElementById("itemform_post_content"), snippet, '', 1, window.opener.document );
			self.close();
			return true;
		}

	}
	// -->
	</script>
	<?php
	}}}
?>
<!-- End of detailed file list but not the end of html comments in our code, obviously ;) -->
