<?php
/**
 * This file implements upgrading of DB tables
 *
 * Quam Plures - {@link http://quamplures.net/}
 * Released under GNU GPL License - {@link http://quamplures.net/license.html}
 * @copyright (c) 2009 - 2011 by the Quam Plures developers - {@link http://quamplures.net/}
 * @copyright (c)2003-2009 by Francois PLANQUE - {@link http://fplanque.net/}
 *
 * {@internal Open Source relicensing agreement:
 * Daniel HAHLER grants Francois PLANQUE the right to license
 * Daniel HAHLER's contributions to this file and the b2evolution project
 * under any OSI approved OSS license (http://www.opensource.org/licenses/).
 * }}
 *
 * @package pond
 */
if( !defined('QP_MAIN_INIT') ) die( 'Please, do not access this page directly.' );

load_funcs('_core/_param.funcs.php');

/**
 * Converts languages in a given table into according locales
 *
 * @param string name of the table
 * @param string name of the column where lang is stored
 * @param string name of the table's ID column
 */
function convert_lang_to_locale( $table, $columnlang, $columnID )
{
	global $DB, $locales, $default_locale;

	if( !preg_match('/[a-z]{2}-[A-Z]{2}(-.{1,14})?/', $default_locale) )
	{ // we want a valid locale
		$default_locale = 'en-EU';
	}

	echo 'Converting langs to locales for '. $table. '...<br />';

	// query given languages in $table
	$query = "SELECT $columnID, $columnlang FROM $table";
	$languagestoconvert = array();
	foreach( $DB->get_results( $query, ARRAY_A ) as $row )
	{
		// remember the ID for that locale
		$languagestoconvert[ $row[ $columnlang ] ][] = $row[ $columnID ];
	}

	foreach( $languagestoconvert as $lkey => $lIDs)
	{ // converting the languages we've found
		$converted = false;
		echo '&nbsp; Converting lang \''. $lkey. '\' '; // (with IDs: '. implode( ', ', $lIDs ). ').. ';

		if( preg_match('/[a-z]{2}-[A-Z]{2}(-.{1,14})?/', $lkey) )
		{ // Already valid
			echo 'nothing to update, already valid!<br />';
			continue;
		}

		if( (strlen($lkey) == 2) && ( substr( $default_locale, 0, 2 ) != $lkey ) )
		{ // we have an old two letter lang code to convert
			// and it doesn't match the default locale
			foreach( $locales as $newlkey => $v )
			{ // loop given locales
				if( substr($newlkey, 0, 2) == strtolower($lkey) ) # TODO: check if valid/suitable
				{ // if language matches, update
					$converted = $DB->query( "
						UPDATE $table
						   SET $columnlang = '$newlkey'
						 WHERE $columnlang = '$lkey'" );
					echo 'to locale \''. $newlkey. '\'<br />';
					break;
				}
			}
		}

		if( !$converted )
		{ // we have nothing converted yet, setting default:
			$DB->query( "UPDATE $table
										SET $columnlang = '$default_locale'
										WHERE $columnlang = '$lkey'" );
			echo 'forced to default locale \''. $default_locale. '\'<br />';
		}
	}
	echo "\n";
}


/**
 * Add a column, if it does not already exist.
 * If it exists already, a "ALTER TABLE" statement will get executed instead.
 *
 * @return boolean True if the column has been added, False if not.
 */
function db_add_col( $table, $col_name, $col_desc )
{
	global $DB;

	if( db_col_exists($table, $col_name) )
	{ // Column exists already, make sure it's the same.
		$DB->query( 'ALTER TABLE '.$table.' MODIFY COLUMN '.$col_name.' '.$col_desc );
		return false;
	}

	$DB->query( 'ALTER TABLE '.$table.' ADD COLUMN '.$col_name.' '.$col_desc );
}


/**
 * Add an INDEX. If another index with the same name already exists, it will
 * get dropped before.
 */
function db_add_index( $table, $name, $def )
{
	global $DB;
	if( db_index_exists($table, $name) )
	{
		$DB->query( 'ALTER TABLE '.$table.' DROP INDEX '.$name );
	}
	$DB->query( 'ALTER TABLE '.$table.' ADD INDEX '.$name.' ('.$def.')' );
}


/**
 * @param string Table name
 * @param array Column names
 * @return boolean Does a list of given column names exist in DB?
 */
function db_cols_exist( $table, $col_names )
{
	global $DB;

	foreach( $col_names as $k => $v )
		$col_names[$k] = strtolower($v);

	foreach( $DB->get_results('SHOW COLUMNS FROM '.$table) as $row )
		if( ($key = array_search(strtolower($row->Field), $col_names)) !== false )
			unset( $col_names[$key] );

	return count($col_names) == 0;
}

/**
 * Drops a column, if it exists.
 */
function db_drop_col( $table, $col_name )
{
	global $DB;

	if( ! db_col_exists($table, $col_name) )
		return false;

	$DB->query( 'ALTER TABLE '.$table.' DROP COLUMN '.$col_name );
}


/**
 * @return boolean Does a given index key name exist in DB?
 */
function db_index_exists( $table, $index_name )
{
	global $DB;

	$index_name = strtolower($index_name);

	foreach( $DB->get_results('SHOW INDEX FROM '.$table) as $row )
	{
		if( strtolower($row->Key_name) == $index_name )
		{
			return true;
		}
	}

	return false;
}


/**
 * Create a DB version checkpoint
 *
 * This increments table.db_upgrade during an upgrade cycle. It starts matching
 * table.db_version and end matching $app_db_version. If the upgrader crashes
 * this value lets us pick up exactly where we left off :)
 *
 * @param string version of DB at checkpoint
 */
function set_upgrade_checkpoint( $version )
{
	global $DB, $script_start_time, $locale;

	$DB->query( "UPDATE T_settings SET set_value = '$version' WHERE set_name = 'db_upgrade'" );

	$max_exe_time = ini_get( 'max_execution_time' );
	$elapsed_time = time() - $script_start_time;
	if( $max_exe_time && $elapsed_time > $max_exe_time - 10 )
	{ // Max exe time not disabled and we're reaching the end
		echo 'We are reaching the time limit for this script. Please click <a href="index.php?locale='.$locale.'&amp;action=upgradedb">continue</a>...';
		// Dirty temporary solution:
		exit(0);
	}

	task_end();

}


/**
 * Upgrade database tables
 *
 * This is a process. Life is a process. Therefore this is life.
 */
function upgrade_dbase_tables()
{
	global $old_db_version;
	global $DB;
	global $app_name;
	global $app_db_version;
	global $schema_queries;

	// Load DB schema from modules
	load_db_schema();
	load_funcs('_core/model/db/_upgrade.funcs.php');

	// BEGIN installation status checks...
	echo '<p>'.T_('Checking DB schema version...').' ';

	// are we not installed?
	$old_db_version = get_db_version( 'version' );
	if( $old_db_version === NULL )
	{
		printf( T_( '<strong>OOPS! %s doesn\'t seem to be installed yet.</strong></p>' ), $app_name );
		return;
	}

	// are we too old to rock and roll or too young to die?
	echo $old_db_version, ' : ';
	if( $old_db_version < 0 )
	{
		debug_die( T_('This version is too old!') );
	}
	if( $old_db_version > $app_db_version )
	{
		debug_die( T_('This version is too recent! We cannot downgrade to the version you are trying to install...') );
	}

	// did we crash while upgrading?
	$cur_db_version = get_db_version( 'upgrade' );
	if( $cur_db_version === NULL )
	{	// Initialize current DB version (upgrade process):
		$cur_db_version = $old_db_version;
		$DB->query( "INSERT INTO T_settings (set_name, set_value) VALUES ('db_upgrade', '0')" );
	}
	else if( $cur_db_version != $old_db_version )
	{
		echo T_('It looks like we crashed while upgrading. We will try to pick up where we left off').'<br />';
		echo sprintf( T_('You had version %s and crashed at version %s on the way to version %s.'), $old_db_version, $cur_db_version, $app_db_version ).' ... ';
	}

	echo "OK.<br />\n";
	// END installation status checks...

	// Try to obtain some serious time to do some serious processing (5 minutes and 57 seconds)
	@set_time_limit( 357 );

	// ---------------------------------------------------------------------------
	// BEGIN UPGRADES FROM V0.0.0 ($app_db_version = 0)
	// ---------------------------------------------------------------------------

	// ---------------------------------------------------------------------------
	// BEGIN BRANCH: html5_step_one
	// https://code.launchpad.net/~edb/quam-plures/html5_step_one
	// ---------------------------------------------------------------------------

	if( $old_db_version < 1 )
	{	// change the template_type options...
		echo 'Upgrading templates__template table (1 of 4) ... ';
		db_add_col( "T_templates__template", "template_type", "ENUM('xhtml', 'normal', 'feed', 'other') NOT NULL DEFAULT 'xhtml'" );
		set_upgrade_checkpoint( '1' );
	}

	if( $old_db_version < 2 )
	{	// change "normal" types to "xhtml"
		echo 'Upgrading templates__template table (2 of 4) ... ';
		$DB->query( "UPDATE T_templates__template SET template_type = REPLACE( template_type, 'normal', 'xhtml' )" );
		set_upgrade_checkpoint( '2' );
	}

	if( $old_db_version < 3 )
	{	// if Sitemap is installed change the type to "other"
		echo 'Upgrading templates__template table (3 of 4) ... ';
		$DB->query( "UPDATE T_templates__template SET template_type = 'other' WHERE template_name = 'Sitemap'" );
		set_upgrade_checkpoint( '3' );
	}

	if( $old_db_version < 4 )
	{	// change the template_type options again...
		echo 'Upgrading templates__template table (4 of 4) ... ';
		db_add_col( "T_templates__template", "template_type", "ENUM('xhtml', 'feed', 'other') NOT NULL DEFAULT 'xhtml'" );
		set_upgrade_checkpoint( '4' );
	}

	// ---------------------------------------------------------------------------
	// END BRANCH: html5_step_one
	// ---------------------------------------------------------------------------

	/*****************************************************************************
	 * ADD UPGRADES HERE!                                                        *
	 * Remember changing the DB schema, too! This provides for fully automatic   *
	 * upgrades when there were changes but the DB version seems to be           *
	 * up-to-date. Look at the following files:                                  *
	 *   qp_inc/_core/model/__core.install.php                                   *
	 *   qp_inc/sessions/model/_sessions.install.php                             *
	 *****************************************************************************/

	// ---------------------------------------------------------------------------
	// THE FOLLOWING STUFF GETS DONE AFTER ALL THE DATABASE CHANGES ARE DONE
	// ---------------------------------------------------------------------------

	// set db_version to match what our last upgrade value said we were at
	$end_of_upgrade = get_db_version( 'upgrade' );
	$DB->query( "UPDATE T_settings SET set_value = '$end_of_upgrade' WHERE set_name = 'db_version'" );

	// double-check our upgrade against our target
	if( $end_of_upgrade != $app_db_version )
	{	// This is a serious problem that should never reach production!
		// It means that our branch-work wasn't done well, so fix it :)
		debug_die( 'The end of the upgrade process did not get us where we needed to be! '.$end_of_upgrade.' vs '.$app_db_version );
	}

	// Check to make sure the DB schema is up to date:
	$upgrade_db_deltas = array(); // This holds changes to make, if any (just all queries)
	foreach( $schema_queries as $table => $query_info )
	{ // For each table in the schema, check diffs...
		foreach( db_delta( $query_info[1], array('drop_column', 'drop_index') ) as $table => $queries )
		{
			foreach( $queries as $qinfo )
			{
				foreach( $qinfo['queries'] as $query )
				{ // subqueries for this query (usually one, but may include required other queries)
					$upgrade_db_deltas[] = $query;
				}
			}
		}
	}

	if( empty($upgrade_db_deltas) )
	{
		echo '<p>'.T_('The database schema is up to date.').'</p>';
	}
	else
	{
		// delta queries have to be confirmed or executed now..
		$confirmed_db_upgrade = param('confirmed', 'integer', 0); // force confirmation
		$upgrade_db_deltas_confirm_md5 = param( 'upgrade_db_deltas_confirm_md5', 'string', '' );

		if( ! $confirmed_db_upgrade )
		{
			if( ! empty($upgrade_db_deltas_confirm_md5) )
			{ // received confirmation from form
				if( $upgrade_db_deltas_confirm_md5 != md5( implode('', $upgrade_db_deltas) ) )
				{ // unlikely to happen
					echo '<p class="error">'
						.T_('The DB schema has been changed since confirmation.')
						.'</p>';
				}
				else
				{
					$confirmed_db_upgrade = true;
				}
			}
		}

		if( ! $confirmed_db_upgrade )
		{
			global $action, $locale;
			load_class( '_core/ui/forms/_form.class.php' );
			$Form = new Form( NULL, '', 'post' );
			$Form->begin_form( 'fform', T_('Upgrade database') );
			$Form->begin_fieldset();
			$Form->hidden( 'upgrade_db_deltas_confirm_md5', md5(implode( '', $upgrade_db_deltas )) );
			$Form->hidden( 'action', $action );
			$Form->hidden( 'locale', $locale );
			echo '<p>'.T_('The version number is correct, but we have detected changes in the database schema. This can happen with <abbr title="Version Control System">VCS</abbr> versions...').'</p>';
			echo '<p>'.T_('The following database changes will be carried out. If you are not sure what this means, it will probably be alright.').'</p>';
			echo '<ul>';
			foreach( $upgrade_db_deltas as $l_delta )
			{
				#echo '<li><code>'.nl2br($l_delta).'</code></li>';
				echo '<li><pre>'.str_replace( "\t", '  ', $l_delta ).'</pre></li>';
			}
			echo '</ul>';
			$Form->submit( array( '', T_('Upgrade database!'), 'ActionButton' ) );
			$Form->end_form();

			return false;
		}

		// Alter DB to match DB schema:
		install_make_db_schema_current( true );
	}

	return true;
}

?>
