<?php
/**
 * This plugin replaces the textarea in the "Write" tab with {@link http://tinymce.moxiecode.com/ tinyMCE}.
 *
 * This file is part of the Quam Plures project - {@link http://quamplures.net/}.
 * See also {@link https://launchpad.net/quam-plures}.
 *
 * @copyright 2006 by Daniel HAHLER - {@link http://daniel.hahler.de/}.
 * @copyright 2009 by Francois Planque - {@link http://fplanque.com/}.
 *
 * @license http://quamplures.net/license.html GNU General Public License (GPL)
 *
 * @author blueyed: Daniel HAHLER
 * @author fplanque: Francois Planque
 * @author PhiBo: Philipp Seidel (since version 0.6)
 *
 * @package plugins
 * @subpackage TinyMCE
 */
if( !defined('QP_MAIN_INIT') ) die( 'Please, do not access this page directly.' );


/**
 * @package plugins
 * @subpackage TinyMCE
 */
class tinymce_plugin extends Plugin
{
	var $code = 'tinymce';
	var $group = 'other';
	var $sub_group = 'editor'; // used if $group is 'other'
	var $number_of_installs = 1;
	var $priority = 10;
	var $version = '0.1';

	// internal use vars...
	var $tmce_editor_id = 'itemform_post_content';


	/**
	 * @see Plugin::PluginInit()
	 */
	function PluginInit( & $params )
	{
		$this->name = $this->T_('TinyMCE');
		$this->short_desc = $this->T_('Javascript-powered WYSIWYG editor');
		$this->long_desc = $this->T_('Provides a javascript-powered "WYSIWYG" editor. Note that this editor sometimes makes "bad" code. If validation is important to you then you might be unhappy.');
	}


	/**
	 * @see Plugin::GetDefaultSettings()
	 */
	function GetDefaultSettings( & $params )
	{
		$plain_text_only = '<strong>'.T_('plain text only').'</strong>';
		$html_allowed = '<strong class="red">'.T_('html allowed').'</strong>';
		$default_settings = array(
			'default_use_tinymce' => array(
				'label' => $this->T_('Use TinyMCE (default)'),
				'type' => 'checkbox',
				'defaultvalue' => '0',
				'note' => $this->T_('This is the default, which users can override in their profile.'),
			),
			'tmce_options_directionality' => array(
				'label' => $this->T_('Directionality support'),
				'type' => 'checkbox',
				'defaultvalue' => 0,
				'note' => $this->T_('Enable to add directionality icons to TinyMCE for better handling of right-to-left languages')
			),
			'tmce_custom_conf' => array(
				'label' => $this->T_('Custom TinyMCE init'),
				'type' => 'textarea',
				'defaultvalue' => // Provide some sample:
						'theme_advanced_toolbar_location : "top", '."\n"
						.'height : "400"',
				'note' => sprintf( $this->T_('Custom parameters to tinyMCE.init(). See the <a %s>TinyMCE manual</a>.'), 'href="http://wiki.moxiecode.com/index.php/TinyMCE:Configuration"' ),
			),
			'primary_start' => array(
				'label' => $this->T_('Insert Image Settings'),
				'layout' => 'begin_fieldset',
			),
			'primary_info' => array(
				'label' => $this->T_('WARNING'),
				'type' => 'info',
				'info' => $this->T_('<strong>DO NOT CHANGE THE NEXT 20 SETTINGS IF YOU ARE NOT GOOD WITH HTML!!!</strong><ul><li>For all settings: if you enter "FLOATDIRECTION" (no quote marks) it will be replaced with "floatleft" or "floatright" depending on which way you want to float an image. The primary purpose is for a "class" attribute.</li><li>For all settings: if you enter "STYLEWIDTH" (no quote marks) it will be replaced with "width:XXXpx;", where XXX is the image\'s width plus 12. The primary purpose is for a "style" attribute.</li><li>For all settings: if you enter "WIDTHONLY" (no quote marks) it will be replaced with "XXX", where XXX is the image\'s width plus 12. The primary purpose is for a "width" attribute for a table (TinyMCE).</li><ul>'),
			),
			'primary_info_b' => array(
				'label' => $this->T_('Info'),
				'type' => 'info',
				'info' => $this->T_('These settings appear in roughly the order they are used to help make sense of it all.<ul><li>Some settings are marked either "(floats)" or "(centered)". If we need different values for floating or centering we do it, otherwise the setting is shared across both methods.</li><li>ONLY "image class" and "image style" are used when inserting an image inline.</li></ul>'),
			),
			'overallblock_start' => array(
				'label' => $this->T_('overall html block settings'),
				'layout' => 'begin_fieldset',
			),
			'overallblock_info' => array(
				'label' => $this->T_('Info'),
				'type' => 'info',
				'info' => $this->T_('These are for the entire inserted html block (title if present, image, and caption if present).'),
			),
			'insert_tag' => array(
				'label' => $this->T_('insert tag'),
				'type' => 'text',
				'size' => 64,
				'defaultvalue' => 'table align="center" width="WIDTHONLY"',
				'note' => '<br />'.$plain_text_only,
			),
			'insert_class_floats' => array(
				'label' => $this->T_('insert class (floats)'),
				'type' => 'text',
				'size' => 64,
				'defaultvalue' => 'FLOATDIRECTION',
				'note' => '<br />'.$plain_text_only,
			),
			'insert_class_centered' => array(
				'label' => $this->T_('insert class (centered)'),
				'type' => 'text',
				'size' => 64,
				'defaultvalue' => 'image_block center',
				'note' => '<br />'.$plain_text_only,
			),
			'insert_style_floats' => array(
				'label' => $this->T_('insert style (floats)'),
				'type' => 'text',
				'size' => 64,
				'defaultvalue' => '',
				'note' => '<br />'.$plain_text_only,
			),
			'insert_style_centered' => array(
				'label' => $this->T_('insert style (centered)'),
				'type' => 'text',
				'size' => 64,
				'defaultvalue' => '',
				'note' => '<br />'.$plain_text_only,
			),
			'overallblock_end' => array( 'layout' => 'end_fieldset' ),

			'titlesection_start' => array(
				'label' => $this->T_('title section settings'),
				'layout' => 'begin_fieldset',
			),
			'titlesection_info' => array(
				'label' => $this->T_('Info'),
				'type' => 'info',
				'info' => $this->T_('These are used only if the title is displayed ("default" and a title exists, or "placeholder").'),
			),
			'title_tag' => array(
				'label' => $this->T_('title tag'),
				'type' => 'text',
				'size' => 64,
				'defaultvalue' => 'caption',
				'note' => '<br />'.$plain_text_only,
			),
			'title_class' => array(
				'label' => $this->T_('title class'),
				'type' => 'text',
				'size' => 64,
				'defaultvalue' => 'image_title',
				'note' => '<br />'.$plain_text_only,
			),
			'title_style' => array(
				'label' => $this->T_('title style'),
				'type' => 'text',
				'size' => 64,
				'defaultvalue' => '',
				'note' => '<br />'.$plain_text_only,
			),
			'title_post' => array(
				'label' => $this->T_('post-title'),
				'type' => 'html_textarea', // allows "<"
				'rows' => 1,
				'cols' => 60,
				'defaultvalue' => '',
				'note' => $this->T_('added after closing the "title tag" ONLY IF there is a title').' &middot; '.$html_allowed,
			),
			'titlesection_end' => array( 'layout' => 'end_fieldset' ),
			'imagesection_start' => array(
				'label' => $this->T_('image section settings'),
				'layout' => 'begin_fieldset',
			),
			'imagesection_info' => array(
				'label' => $this->T_('Info'),
				'type' => 'info',
				'info' => $this->T_('These settings affect the actual IMG tag, including possible code before and after the image tag.'),
			),
			'imagesection_pre_start' => array(
				'label' => $this->T_('before the image tag'),
				'layout' => 'begin_fieldset',
			),
			'imagesection_pre_info' => array(
				'label' => $this->T_('Info'),
				'type' => 'info',
				'info' => $this->T_('This is used between the title section and the image tag, even if there is no title section displayed.'),
			),
			'image_pre' => array(
				'label' => $this->T_('pre-image'),
				'type' => 'html_textarea', // allows "<"
				'rows' => 1,
				'cols' => 60,
				'defaultvalue' => '<tbody><tr><td>',
				'note' => $html_allowed,
			),
			'imagesection_pre_end' => array( 'layout' => 'end_fieldset' ),
			'imagesection_attribs_start' => array(
				'label' => $this->T_('image tag attributes'),
				'layout' => 'begin_fieldset',
			),
			'image_class' => array(
				'label' => $this->T_('image class'),
				'type' => 'text',
				'size' => 64,
				'defaultvalue' => 'center',
				'note' => '<br />'.$this->T_('class value for the IMG tag').' &middot; '.$plain_text_only,
			),
			'image_style' => array(
				'label' => $this->T_('image style'),
				'type' => 'text',
				'size' => 64,
				'defaultvalue' => '',
				'note' => '<br />'.$this->T_('style value for the IMG tag').' &middot; '.$plain_text_only,
			),
			'imagesection_attribs_end' => array( 'layout' => 'end_fieldset' ),
			'imagesection_post_start' => array(
				'label' => $this->T_('after the image tag'),
				'layout' => 'begin_fieldset',
			),
			'imagesection_post_info' => array(
				'label' => $this->T_('Info'),
				'type' => 'info',
				'info' => $this->T_('This is used between the image tag and the caption section, even if there is no caption section displayed.'),
			),
			'image_post' => array(
				'label' => $this->T_('post-image'),
				'type' => 'html_textarea', // allows "<"
				'rows' => 1,
				'cols' => 60,
				'defaultvalue' => '',
				'note' => $html_allowed,
			),
			'imagesection_post_end' => array( 'layout' => 'end_fieldset' ),
			'imagesection_end' => array( 'layout' => 'end_fieldset' ),
			'captionsection_start' => array(
				'label' => $this->T_('caption section settings'),
				'layout' => 'begin_fieldset',
			),
			'captionsection_info' => array(
				'label' => $this->T_('Info'),
				'type' => 'info',
				'info' => $this->T_('These are used only if the caption is displayed ("default" and a caption exists, or "placeholder").'),
			),
			'caption_pre' => array(
				'label' => $this->T_('pre-caption'),
				'type' => 'html_textarea', // allows "<"
				'rows' => 1,
				'cols' => 60,
				'defaultvalue' => '</td></tr><tr>',
				'note' => $this->T_('before the caption section tag').' &middot; '.$html_allowed,
			),
			'caption_tag' => array(
				'label' => $this->T_('caption tag'),
				'type' => 'text',
				'size' => 64,
				'defaultvalue' => 'td',
				'note' => '<br />'.$plain_text_only,
			),
			'caption_class' => array(
				'label' => $this->T_('caption class'),
				'type' => 'text',
				'size' => 64,
				'defaultvalue' => 'image_legend',
				'note' => '<br />'.$plain_text_only,
			),
			'caption_style' => array(
				'label' => $this->T_('caption style'),
				'type' => 'text',
				'size' => 64,
				'defaultvalue' => '',
				'note' => '<br />'.$plain_text_only,
			),
			'caption_post' => array(
				'label' => $this->T_('post-caption'),
				'type' => 'html_textarea', // allows "<"
				'rows' => 1,
				'cols' => 60,
				'defaultvalue' => '',
				'note' => $this->T_('used after closing the caption tag').' &middot; '.$html_allowed,
			),
			'nocaption_start' => array(
				'label' => $this->T_('no caption displayed'),
				'layout' => 'begin_fieldset',
			),
			'nocaption_info' => array(
				'label' => $this->T_('Info'),
				'type' => 'info',
				'info' => $this->T_('This is used only if there is no caption present (useful for TinyMCE)'),
			),
			'caption_none' => array(
				'label' => $this->T_('NO caption'),
				'type' => 'html_textarea', // allows "<"
				'rows' => 1,
				'cols' => 60,
				'defaultvalue' => '</td>',
				'note' => $html_allowed,
			),
			'nocaption_end' => array( 'layout' => 'end_fieldset' ),
			'captionsplitter_start' => array(
				'label' => $this->T_('Caption -> Closing separator'),
				'layout' => 'begin_fieldset',
			),
			'captionsplitter_info' => array(
				'label' => $this->T_('Info'),
				'type' => 'info',
				'info' => $this->T_('This is used between the caption section and the overall block closing, even if there is no caption section displayed.'),
			),
			'caption_separator' => array(
				'label' => $this->T_('post caption separator'),
				'type' => 'html_textarea', // allows "<"
				'rows' => 1,
				'cols' => 60,
				'defaultvalue' => '</tr></tbody>',
				'note' => $html_allowed,
			),
			'captionsplitter_end' => array( 'layout' => 'end_fieldset' ),
			'captionsection_end' => array( 'layout' => 'end_fieldset' ),
		);
		return $default_settings;
	}


	/**
	 * @see Plugin::GetExtraEvents()
	 *
	 * Plugins can set the "load_before_init" parameter with some javascript code
	 * that will be executed before tinyMCE.init() is called. This is most useful
	 * for inserting code to load an external tinyMCE plugin.
	 *
	 * Supported events are as follows:
	 * tinymce_before_init:
	 * Allows other QP plugins to load tinymce plugins before the tinymce init.
	 * Example:
	 * <code>function tinymce_before_init( &$params ) {
	 *   $mypluginurl = \$this->get_plugin_url()."myplugin/editor_plugin.js";
	 *   echo "tinymce.PluginManager.load('myplugin', '".$mypluginurl."')";
	 * }</code>
	 *
	 * tinymce_extend_plugins:
	 * Allows QP plugins to extend the plugin list. TinyMCE often needs to be told not to
	 * load an external plugin during it's load phase because it's already been loaded.
	 * The plugin list is exposed in the tinymce_plugins property in the params.
	 * Example:
	 * <code>function tinymce_extend_plugins( &$params ) {
	 *   array_push($params["tinymce_plugins"], "-myplugin");
	 * }</code>
	 *
	 * tinymce_extend_buttons:
	 * Allows QP plugins to extend the buttons in the third button panel. The
	 * buttons list is exposed in the tinymce_buttons property in the params.
	 * Example:
	 * <code>function tinymce_extend_buttons( &$params ) {
	 *   array_push($params["tinymce_buttons"], "mypluginbutton");
	 * }</code>
	 */
	function GetExtraEvents()
	{
		return array(
			"tinymce_before_init"    => "Event that is called before tinymce is initialized",
			"tinymce_extend_plugins" => "Event called to allow other plugins to extend the plugin list",
			"tinymce_extend_buttons" => "Event called to allow other plugins to extend the button list"
		);
	}


	/**
	 * @see Plugin::GetDefaultUserSettings()
	 */
	function GetDefaultUserSettings( & $params )
	{
		$r = array(
			'use_tinymce' => array(
				'label' => $this->T_('Use TinyMCE'),
				'type' => 'checkbox',
				'defaultvalue' => $this->Settings->get('default_use_tinymce'),
				'note' => $this->T_('Check this to enable the extended Javascript editor (TinyMCE).'),
			),
		);
		$r['tmce_options_directionality'] = array(
					'label' => $this->T_('Directionality support'),
					'type' => 'checkbox',
					'defaultvalue' => $this->Settings->get('tmce_options_directionality'),
					'note' => $this->T_('Enable to add directionality icons to TinyMCE that enables TinyMCE to better handle languages that are written from right to left.')
				);
		return $r;
	}


	/**
	 * Get the URL to include TinyMCE.
	 * @return string
	 */
	function get_tinymce_src_url()
	{
		$base_path = $this->get_plugin_url().'tiny_mce/';
		$url = $base_path.'tiny_mce.js';
		// Append version string, so it gets updated on updates.
		$url = url_add_param($url, 'v='.$this->version);
		return $url;
	}


	/**
	 * @see Plugin::AdminDisplayEditorButton()
	 */
	function AdminDisplayEditorButton( & $params )
	{
		global $Blog;
		/**
		 * @global Item
		 */
		global $edited_Item;

		if( $params['target_type'] != 'Item' )
		{ // only for Items:
			return;
		}

		if( ! $this->UserSettings->get('use_tinymce') )
		{
			// no button for wysiwyg if the user doesn't want it
			return;
		}

		/**
		* @todo EdB: forcing expert to do away with simple but 'edit_layout' is still around (and still valuable?)
		*
		* <input id="tinymce_plugin_toggle_button" type="button" value="WYSIWYG" style="display:none"
		*  title="<?php echo htmlspecialchars($this->T_('Toggle between WYSIWYG and plain HTML editor')); ?>" />
		*
		*/
		$tmce_init = $this->get_tmce_init( 'expert' );
		?>

<!-- <input> (above in comment block) removed from here -->

		<script type="text/javascript">
			jQuery("#tinymce_plugin_toggle_button").click( function() {
				tinymce_plugin_toggleEditor('<?php echo $this->tmce_editor_id; ?>'); } );
			/**
			 * Toggle TinyMCE editor on/off.
			 * This updates the corresponding PluginUserSetting, too.
			 */
			function tinymce_plugin_toggleEditor(id)
			{
				jQuery('#tinymce_plugin_toggle_button').attr("disabled", "disabled");
				if( ! tinymce_plugin_init_done )
				{
					tinymce_plugin_init_done = true;
					// call this method on init again, with "null" id, so that mceAddControl gets called.
					tinymce_plugin_load_tinymce( function() {tinymce_plugin_toggleEditor(null)} );
					return;
				}

				if( ! tinyMCE.getInstanceById(id))
				{
					tinyMCE.execCommand('mceAddControl', false, id);
					jQuery.get('<?php echo $this->get_srvc_url('save_editor_state', array('on'=>1, 'blog'=>$Blog->ID, 'item'=>$edited_Item->ID), '&'); ?>');
					jQuery('#tinymce_plugin_toggle_button').attr('value', 'HTML');
				}
				else
				{
					tinyMCE.execCommand('mceRemoveControl', false, id);
					jQuery.get('<?php echo $this->get_srvc_url('save_editor_state', array('on'=>0, 'blog'=>$Blog->ID, 'item'=>$edited_Item->ID), '&'); ?>');
					jQuery('#tinymce_plugin_toggle_button').attr('value', 'WYSIWYG');
				}
				jQuery('#tinymce_plugin_toggle_button').removeAttr("disabled");
			}

			// Make the "toggle" button visible using JS:
			jQuery('#tinymce_plugin_toggle_button').css('display', '');
			var tmce_init={<?php echo $tmce_init; ?>};
			var tinymce_plugin_displayed_error = false;
			var tinymce_plugin_init_done = false;
			</script>

			<?php
			// Load TinyMCE Javascript source file:
			// This cannot be done through AJAX, since there appear to be scope problems on init then (TinyMCE problem?! - "u not defined").
			// Anyway, not using AJAX to fetch the file makes it more cachable anyway.
			echo '<script type="text/javascript" src="'.htmlspecialchars($this->get_tinymce_src_url()).'"></script>';
			?>

			<script type="text/javascript">
			/**
			 * Javascript function to load and init TinyMCE.
			 * This gets done dynamically, either "on loading" or by AJAX, if the toggle button enables the editor.
			 * @param function to call after init
			 */
			function tinymce_plugin_load_tinymce( oninit )
			{
				tinymce_plugin_init_tinymce(oninit);
			}

			function tinymce_plugin_init_tinymce(oninit)
			{
				// Init tinyMCE:
				if( typeof tinyMCE == "undefined" )
				{
					if( ! tinymce_plugin_displayed_error )
					{
						alert( '<?php echo str_replace("'", "\'",
							sprintf( $this->T_('TinyMCE javascript could not be loaded. Check the "%s" plugin setting.'),
							$this->T_('URL to TinyMCE') ) ) ?>' );
						tinymce_plugin_displayed_error = true;
					}
				}
				else
				{
					<?php
					global $Plugins;
					$Plugins->trigger_event('tinymce_before_init');
					?>

					// Define oninit function for TinyMCE
					if( typeof tmce_init.oninit != "undefined" )
					{
						oninit = function() {
							tmce_init.oninit();
							oninit();
						}
					}

					tmce_init.oninit = function ()
					{
						oninit();

						// Provide hooks for textarea manipulation (where other plugins should hook into):
						var ed = tinyMCE.get("<?php echo $this->tmce_editor_id ?>");
						if( ed && typeof b2evo_Callbacks == "object" )
						{
							// add a callback, that returns the selected (raw) html:
							b2evo_Callbacks.register_callback( "get_selected_text_for_itemform_post_content", function(value) {
									var inst = tinyMCE.get("<?php echo $this->tmce_editor_id ?>");
									if( ! inst ) return null;
									return inst.selection.getContent();
								} );

							// add a callback, that wraps a selection:
							b2evo_Callbacks.register_callback( "wrap_selection_for_itemform_post_content", function(params) {
									var inst = tinyMCE.get("<?php echo $this->tmce_editor_id ?>");
									if( ! inst ) return null;
									var sel = inst.selection.getContent();

									if( params.replace )
									{
										var value = params.before + params.after;
									}
									else
									{
										var value = params.before + sel + params.after;
									}
									inst.selection.setContent(value);

									return true;
								} );

							// add a callback, that lets us insert raw content:
							// DEPRECATED, used in b2evo 1.10.x
							b2evo_Callbacks.register_callback( "insert_raw_into_itemform_post_content", function(value) {
									tinyMCE.execInstanceCommand( "<?php echo $this->tmce_editor_id ?>", "mceInsertRawHTML", false, value );
									return true;
							} );
						}
					}
					tinyMCE.init(tmce_init);
				}
			}
		</script>

		<?php
		if( is_object($edited_Item) && !empty($edited_Item->editor_code) )
		{ // We have a preference for the current post, follow it:
			// Use tinyMCE if code matched the code of the current plugin.
			// fp> Note: this is a temporary solution; in the long term, this will be part of the API and the appropriate plugin will be selected.
			$use_tinymce = ($edited_Item->editor_code == $this->code);
		}
		else
		{ // We have no pref, fall back to whatever current user has last used:

			// Has the user used MCE last time he edited this particular blog?
			$use_tinymce = $this->UserSettings->get('use_tinymce_coll'.$Blog->ID );

			if( is_null($use_tinymce) )
			{ // We don't know for this blog, check if he used MCE last time he edited anything:
				$use_tinymce = $this->UserSettings->get('use_tinymce');
			}
		}

		if( $use_tinymce )
		{ // User used MCE last time, load MCE now:
			// fp> why do we need to wrap in jQuery() here?
			echo '<script type="text/javascript">jQuery( tinymce_plugin_toggleEditor("'.$this->tmce_editor_id.'") );</script>';
		}

		// We also want to save the 'last used/not-used' state: (if no NULLs, this won't change anything)
		$this->srvc_save_editor_state( array('on'=>$use_tinymce, 'blog'=>$Blog->ID, 'item'=>$edited_Item->ID ) );

		return true;
	}


	/**
	 * Create Options for TinyMCE.init() (non-compressor) - not TinyMCE_GZ.init (compressor)!!
	 *
	 * @todo fp> valid_elements to try to generate less validation errors
	 *
	 * @param string simple|expert
	 * @return string|false
	 */
	function get_tmce_init( $edit_layout )
	{
		global $Blog;
		global $Plugins;
		global $localtimenow, $debug, $rsc_url, $templates_url;
		global $UserSettings;

		$tmce_plugins_array = array( 'more', 'noteaser', 'pagebreak', 'searchreplace', 'inlinepopups', 'table', 'media', 'visualchars', 'nonbreaking', 'safari', 'fullscreen, autosave, contextmenu' );
		$tmce_theme_advanced_buttons1_array = array();
		$tmce_theme_advanced_buttons2_array = array();
		$tmce_theme_advanced_buttons3_array = array();

		/* ----------- button row 1 ------------ */

		$tmce_theme_advanced_buttons1_array = array(
			'fontselect,|,fontsizeselect,|,formatselect,|,styleselect',
			'justifyleft,justifycenter,justifyright,justifyfull',
			'bold,italic,strikethrough,|,forecolor,backcolor'
		);

		/* ----------- button row 2 ------------ */

		$tmce_theme_advanced_buttons2_array = array(
			'nonbreaking,|,charmap,|,image,|,media',
			'link,unlink',
			'bullist,numlist,outdent,indent',
			'sub,sup',
			'morebtn,|,nt_btn,|,pagebreak',
			'search,|,replace,|,undo,|,redo',
			'removeformat',
			'visualchars,visualaid',
			'fullscreen'
		);

		/* ----------- button row 3 ------------ */

		$tmce_theme_advanced_buttons3_array = array(
			'table,row_props,cell_props,delete_col,delete_row,col_after,col_before,row_after,row_before,row_after,row_before,split_cells,merge_cells'
		);

		if( $this->UserSettings->get('tmce_options_directionality') == 1 )
		{
			$tmce_plugins_array[] = 'directionality';
			array_push($tmce_theme_advanced_buttons3_array, 'ltr,rtl');
		}

		$tmce_plugins_array[] = 'paste';
		$tmce_theme_advanced_buttons3_array[] = 'pastetext,pasteword';

		array_push($tmce_plugins_array,'xhtmlxtras');
		array_push($tmce_theme_advanced_buttons3_array,'cite,abbr,acronym,del,ins');

		$tmce_theme_advanced_buttons3_array = $Plugins->get_trigger_event("tinymce_extend_buttons",
				array("tinymce_buttons" => $tmce_theme_advanced_buttons3_array),
					"tinymce_buttons");

		$tmce_theme_advanced_buttons3_array[] = 'code,cleanup,|,help';

		$tmce_theme_advanced_buttons1 = implode( ',|,' , $tmce_theme_advanced_buttons1_array );
		$tmce_theme_advanced_buttons2 = implode( ',|,' , $tmce_theme_advanced_buttons2_array );
		$tmce_theme_advanced_buttons3 = implode( ',|,' , $tmce_theme_advanced_buttons3_array );

		// PLUGIN EXTENSIONS:
		$tmce_plugins_array =
			$Plugins->get_trigger_event("tinymce_extend_plugins",
				array("tinymce_plugins" => $tmce_plugins_array),
					"tinymce_plugins");

		$tmce_plugins = implode( ',' , $tmce_plugins_array );

		global $current_locale, $plugins_path;
		$tmce_language = substr($current_locale, 0, 2);
		// waltercruz> Fallback to english if there's no tinymce equivalent to the user locale
		// to avoid some strange screens like http://www.flickr.com/photos/waltercruz/3390729964/
		$lang_path = $plugins_path.$this->classname.'/tiny_mce/langs/'.$tmce_language.'.js';
		if( !file_exists( $lang_path ) )
		{
			$tmce_language = 'en';
		}

		// Configuration: -- http://wiki.moxiecode.com/index.php/TinyMCE:Configuration
		$init_options = array();
		// Convert one specifc textarea to use TinyMCE:
		$init_options[] = 'mode : "exact"';
		$init_options[] = 'elements : "'.$this->tmce_editor_id.'"';
		// TinyMCE Theme+Skin+Variant to use:
		$init_options[] = 'theme : "advanced"';
		$init_options[] = 'skin : "o2k7"';
		$init_options[] = 'skin_variant : "silver"';
		// comma separated list of plugins: -- http://wiki.moxiecode.com/index.php/TinyMCE:Plugins
		$init_options[] = 'plugins : "'.$tmce_plugins.'"';
		$init_options[] = 'more_separator : "<!--more-->"';
		$init_options[] = 'pagebreak_separator : "<!--nextpage-->"';
		// Toolbars:
		$init_options[] = 'theme_advanced_toolbar_location : "top"';
		$init_options[] = 'theme_advanced_toolbar_align : "center"'; // just to be consistent with current toolbars for now
		$init_options[] = 'theme_advanced_buttons1 : "'.$tmce_theme_advanced_buttons1.'"';
		$init_options[] = 'theme_advanced_buttons2 : "'.$tmce_theme_advanced_buttons2.'"';
		$init_options[] = 'theme_advanced_buttons3 : "'.$tmce_theme_advanced_buttons3.'"';
		// UI options:
		$init_options[] = 'theme_advanced_blockformats : "p,pre,blockquote,h2,h3,h4,h5,h6,address,dt,dd,div"';
		if( true )
		{
			$init_options[] = 'theme_advanced_path_location : "bottom"';
			$init_options[] = 'theme_advanced_resizing : true';
			$init_options[] = 'theme_advanced_resize_horizontal : false';
		}
		$init_options[] = 'language : "'.$tmce_language.'"';
		// body_class : "my_class"
		// CSS used in the iframe/editable area: -- http://wiki.moxiecode.com/index.php/TinyMCE:Configuration/content_css
		// note: $version may not be needed below because of automatic suffix? not sure..

		// Load the appropriate ITEM/POST styles depending on the blog's template:
		if( ! empty( $Blog->template_ID) )
		{
			$content_css = ''; // start this empty
			$TemplateCache = & get_Cache( 'TemplateCache' );
			$Template = $TemplateCache->get_by_ID( $Blog->template_ID );
			global $templates_path;
			if( is_file( $templates_path.$Template->folder.'/item.css' ) )
			{
				$item_css_url = $templates_url.$Template->folder.'/item.css';
				$content_css = ','.$item_css_url;
			}
		}

		$init_options[] = 'content_css : "'.$this->get_plugin_url().'editor.css?v='.($debug ? $localtimenow : $this->version )
									.$content_css.'"';
		// Generated HTML code options:
		// do not make the path relative to "document_base_url":
		$init_options[] = 'relative_urls : false';
		$init_options[] = 'entity_encoding : "raw"';

		// remove_linebreaks : false,
		// not documented: auto_cleanup_word : true,

		$init = implode( ",\n", $init_options ).',
			plugin_insertdate_dateFormat : "%Y-%m-%d",
			plugin_insertdate_timeFormat : "%H:%M:%S",

			paste_create_paragraphs : true,
			paste_create_linebreaks : true,
			paste_use_dialog : true,
			paste_convert_headers_to_strong : false,
			paste_convert_middot_lists : true

		';
		// custom conf:
		if( $tmce_custom_conf = $this->Settings->get('tmce_custom_conf') )
		{
			$init .= ",\n// tmce_custom_conf (from PluginSettings):\n".$tmce_custom_conf;
		}
		return $init;
	}


	/**
	 * AJAX callback to save editor state (on or off).
	 */
	function srvc_save_editor_state($params)
	{
		/**
		 * @var DB
		 */
		global $DB;

		if( ! isset($params['on']) )
		{
			return;
		}

		// fp>This is a super dirty hack that we're going to use only until we have a clean API for handling WYSIWYG editors:
		if( !empty($params['item']) )
		{ // Save the status last used by this Item:
			// fp> TODO: also do that on CREATE post because it won't have been done here:
			$sql = 'UPDATE T_items__item
								 SET post_editor_code = '.$DB->quote( (int)$params['on'] ? $this->code : 'html' )
						.' WHERE post_ID = '.$DB->quote((int)$params['item']);
			$DB->query( $sql, 'Save editor state for current post' );
		}

		// Clean:
		if( !empty($params['blog']) )
		{ // This is in order to try & recall a specific state for each blog: (will be used for new posts especially)
			$this->UserSettings->set( 'use_tinymce_coll'.(int)$params['blog'], (int)$params['on'] );
		}
		$this->UserSettings->set( 'use_tinymce', (int)$params['on'] );
		$this->UserSettings->dbupdate();
	}


	/**
	 * @see Plugin::GetSrvcMethods()
	 */
	function GetSrvcMethods()
	{
		return array('save_editor_state');
	}


	/**
	 * @see Plugin::ItemInsertFileJavascript()
	 */
	function ItemInsertFileJavascript()
	{
		if( ! $this->UserSettings->get('use_tinymce') )
		{ // if this user does not use tinymce get out
			return NULL;
		}
		// provide custom tag bits
		return array(
			'insert_tag' => $this->Settings->get('insert_tag'),
			'insert_class_floats' => $this->Settings->get('insert_class_floats'),
			'insert_class_centered' => $this->Settings->get('insert_class_centered'),
			'insert_style_floats' => $this->Settings->get('insert_style_floats'),
			'insert_style_centered' => $this->Settings->get('insert_style_centered'),
			'title_tag' => $this->Settings->get('title_tag'),
			'title_class' => $this->Settings->get('title_class'),
			'title_style' => $this->Settings->get('title_style'),
			'title_post' => $this->Settings->get('title_post'),
			'image_pre' => $this->Settings->get('image_pre'),
			'image_class' => $this->Settings->get('image_class'),
			'image_style' => $this->Settings->get('image_style'),
			'image_post' => $this->Settings->get('image_post'),
			'caption_pre' => $this->Settings->get('caption_pre'),
			'caption_tag' => $this->Settings->get('caption_tag'),
			'caption_class' => $this->Settings->get('caption_class'),
			'caption_style' => $this->Settings->get('caption_style'),
			'caption_post' => $this->Settings->get('caption_post'),
			'caption_none' => $this->Settings->get('caption_none'),
			'caption_separator' => $this->Settings->get('caption_separator'),
		);
	}

}

?>
